﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.ComponentModel;
using Infragistics.Controls.Charts;

namespace SeriesAddingWithCommand
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }
    }

    public class SelectedItemsToSeiresConverter : IMultiValueConverter
    {
        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            XamDataChart chart = values[0] as XamDataChart;

            foreach (Series series in chart.Series)
            {
                series.Visibility = Visibility.Collapsed;
                //(series as ScatterBase).YAxis.LabelSettings.Visibility = Visibility.Collapsed;
            }

            foreach (var item in values[1] as IList<object>)
            {
                if (chart.Series.Where(s => s.DataContext == item).FirstOrDefault() == null)
                {
                    // NumericYAxis yAxis = new NumericYAxis
                    // {
                    //     Label = "{0:N2}-",
                    //     LabelSettings = new AxisLabelSettings()
                    // };

                    ScatterLineSeries series = new ScatterLineSeries
                    {
                        XMemberPath = "X",
                        YMemberPath = "Y",
                        MarkerType = MarkerType.None,
                    };

                    //series.SetBinding(ScatterLineSeries.XAxisProperty, new Binding { Source = chart.Axes.FirstOrDefault() });
                    //series.SetBinding(ScatterLineSeries.YAxisProperty, new Binding { Source = yAxis });
                    series.SetBinding(ScatterLineSeries.XAxisProperty, new Binding { Source = chart.Axes[0] });
                    series.SetBinding(ScatterLineSeries.YAxisProperty, new Binding { Source = chart.Axes[1] });
                    series.SetBinding(ScatterLineSeries.DataContextProperty, new Binding { Source = item });
                    series.SetBinding(ScatterLineSeries.ItemsSourceProperty, new Binding { Path = new PropertyPath("ChartData") });

                    //chart.Axes.Add(yAxis);
                    chart.Series.Add(series);


                    // Console.WriteLine(item)
                }
                else
                {
                    (chart.Series.Where(s => s.DataContext == item).FirstOrDefault() as ScatterBase).Visibility = Visibility.Visible;
                    //(chart.Series.Where(s => s.DataContext == item).FirstOrDefault() as ScatterBase).YAxis.LabelSettings.Visibility = Visibility.Visible;
                }
            }
            return null;
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class MainWindowViewModel : PropertyChangedNotifier
    {
        private ObservableCollection<ChannelViewModel> channals;

        public MainWindowViewModel()
        {
            Channals = new ObservableCollection<ChannelViewModel>();
            Random r = new Random();
            for (int i = 0; i < 5; i++)
            {
                Channals.Add(new ChannelViewModel
                {
                    ChannalName = "Channal " + i,
                    ChartData = ChartDataViewModel.GenerateData()
                });
            }
        }

        public ObservableCollection<ChannelViewModel> Channals
        {
            get { return channals; }
            set { channals = value; OnPropertyChagned("Channals"); }
        }
        
    }

    public class ChannelViewModel : PropertyChangedNotifier
    {
        private string channalName;

        public string ChannalName
        {
            get { return channalName; }
            set { channalName = value; OnPropertyChagned("ChannalName"); }
        }


        private ObservableCollection<ChartDataViewModel> chartData;

        public ObservableCollection<ChartDataViewModel> ChartData
        {
            get { return chartData; }
            set { chartData = value; OnPropertyChagned("ChartData"); }
        }

    }

    public class ChartDataViewModel : PropertyChangedNotifier
    {
        private double x;

        public double X
        {
            get
            {
                return x;
            }
        }

        private double y;

        public double Y
        {
            get { return y; }
            set { y = value; OnPropertyChagned("Value1"); }
        }

        public ChartDataViewModel()
        {

        }

        public ChartDataViewModel(double xval, double yval)
        {
            x = xval;
            y = yval;
        }


        public static ObservableCollection<ChartDataViewModel> GenerateData()
        {
            ObservableCollection<ChartDataViewModel> data = new ObservableCollection<ChartDataViewModel>();


            int noSamples = 65536;
            double lowLevel = -1.0;
            double highLevel = 1.0;
            int noTeeth = 60;
            double widthTooth = 50.0;
            int[] posMissingTeeth = new int[] { 59, 60 };
            double offsetDegree = 0;

            Double actHigh;
            Double actLow;
            Double part1Level;
            Double part2Level;
            Double part1width;
            Double toothInt = 360f / noTeeth;
            Double deg = 0;

            for (UInt16 z = 1; z <= noTeeth * 2; z++)
            {

                if (posMissingTeeth.Contains(z) || posMissingTeeth.Contains((UInt16)(z - noTeeth)))
                {
                    actHigh = lowLevel;
                    actLow = lowLevel;
                }
                else
                {
                    actHigh = highLevel;
                    actLow = lowLevel;
                }

                part1Level = actHigh;
                part2Level = actLow;
                part1width = widthTooth;

                if (z == 1)
                {
                    //Startpoint
                    data.Add(new ChartDataViewModel(deg, part2Level));
                }

                //Part 1 startpoint
                data.Add(new ChartDataViewModel(deg, part1Level));

                //deg forward tooth
                deg += part1width / 100f * toothInt;

                //Over 720 due to m_Delay => go to beginning
                if (deg > 720)
                {

                    //go to start
                    deg -= 720f;

                    //Part 1 endpoint (at 720°)
                    data.Add(new ChartDataViewModel(720, part1Level));

                    //Part 1 endpoint (at 0°)
                    data.Add(new ChartDataViewModel(0, part1Level));

                }

                //Part 1 endpoint
                data.Add(new ChartDataViewModel(deg, part1Level));

                //Part 2 startpoint
                data.Add(new ChartDataViewModel(deg, part2Level));

                //deg forward gap
                deg += (1f - part1width / 100f) * toothInt;

                //Over 720 due to m_Delay => go to beginning
                if (deg > 720)
                {

                    //go to start
                    deg -= 720f;

                    //Part 1 endpoint (at 720°)
                    data.Add(new ChartDataViewModel(720, part2Level));

                    //Part 1 endpoint (at 0°)
                    data.Add(new ChartDataViewModel(0, part2Level));

                }

                //Part 2 endpoint
                data.Add(new ChartDataViewModel(deg, part2Level));

            }

            while (data[data.Count - 1].X <= data[0].X)
            {
                if (data[data.Count - 1].X != data[0].X)
                {
                    data.Insert(0, new ChartDataViewModel(data[data.Count - 1].X, data[data.Count - 1].Y));
                }

                data.RemoveAt(data.Count - 1);
            }

            return data;
        }
    }

    public class PropertyChangedNotifier : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChagned(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }
}
